function [derS derSS R] = BuildDerivativesUniformBS(S,T,V)

% Build the first- and second-order derivatives for the "L" operator matrix
% for the Weighted method
% INPUTS
%    S = vector for uniform stock price grid
%    T = vector for uniform maturity grid
%    V = vector of local volatilities
%    thet = parameter for the weighted scheme
% OUTPUTS
%    Matrices of dimnension N x N
%    derS  = Matrix for first-order derivative dU/dS 
%    derSS = Matrix for second-order derivative dU2/dS2 
%    R     = Matrix for r*S(t) portion of the PDE

% Fabrice Douglas Rouah, FRouah.com and Volopta.com

% Length of stock price, volatility, and maturity
NS = length(S);
NT = length(T);

ds = S(2) - S(1);

Smin = S(1);  Smax = S(NS);
Tmin = T(1);  Tmax = T(NT);

%% Preliminary quantities
% Size of the U(t) vector and L matrix
N = NS*NS;

% The vector for S and V
Si = repmat(S',NS,1);

%% Identification of the boundary points: forward and backward differences
Fs = zeros(N,1);
Bs = zeros(N,1);

I = find(Si==Smin);
J = find(Si==Smax);
Fs(I) = 1;
Bs(J) = 1;

%% Identification of the non boundary points: central differences
Cs = zeros(N,1);  % Central differences
I = find(Fs==0 & Bs==0);
Cs(I) = 1;

%% Initialize the matrices
derS  = zeros(N,N);     % First-order derivative
derSS = zeros(N,N);     % Second-order derivative
R = eye(N);             % for r*S(t) term

%% Interior points: central differences
I = find(Cs==1);
for k=1:length(I)
    % First order derivative
    derS(I(k),I(k)+1) = +1/2/ds * Si(I(k));
    derS(I(k),I(k)-1) = -1/2/ds * Si(I(k));
    % Second order derivative
    derSS(I(k),I(k)+1) = +1/ds^2 * Si(I(k))^2;
    derSS(I(k),I(k))   = -2/ds^2 * Si(I(k))^2;
    derSS(I(k),I(k)-1) = +1/ds^2 * Si(I(k))^2;
end

%% Upper boundary points: forward differences
I = find(Fs==1);
for k=1:length(I)
    % First order derivative
    derS(I(k),I(k))   = -1/2/ds * Si(I(k));
    derS(I(k),I(k)+1) = +1/2/ds * Si(I(k));
    % Second order derivative
    derSS(I(k),I(k))   = +1/ds^2 * Si(I(k))^2;
    derSS(I(k),I(k)+1) = -2/ds^2 * Si(I(k))^2;
    derSS(I(k),I(k)+2) = +1/ds^2 * Si(I(k))^2;
end

%% Lower boundary points: backward differences
I = find(Bs==1);
for k=1:length(I)
    % First order derivative
    derS(I(k),I(k))   = +1/2/ds * Si(I(k));
    derS(I(k),I(k)-1) = -1/2/ds * Si(I(k));
    % Second order derivative
    derSS(I(k),I(k))   = +1/ds^2 * Si(I(k))^2;
    derSS(I(k),I(k)-1) = -2/ds^2 * Si(I(k))^2;
    derSS(I(k),I(k)-2) = +1/ds^2 * Si(I(k))^2;
end


%% Boundary for r*U
I = find(Fs==1);
for k=1:length(I);
	R(I(k),I(k)) = 0;
end
