function WPrice = WeightedMethodBS(thet,S0,Strike,sigma,r,q,S,T,dt,GridType,PutCall,EuroAmer,method)

% Heston Call price using the Weighted Method
% Requires a uniform grid for the stock price, volatility, and maturity
% INPUTS
%   thet = Weighing parameter
%   S0 = Spot price at which to price the call
%   V0 = variance at which to price the call
%   K  = Strike price
%   S = uniform grid for the stock price
%   V = uniform grid for the volatility
%   T = uniform grid for the maturity
%   A = "A" matrix for weighted method
%   invA = A inverse
%   B = "B" matrix for weighted method
% OUTPUT
%   y = 2-D interpolated European Call price

% Fabrice Douglas Rouah, FRouah.com and Volopta.com

% Required vector lengths and time increment
NS = length(S);
NT = length(T);

% Initialize the u vector
u = zeros(NS*NS,1);

% U(0) vector - value of U(T) at maturity
Si = repmat(S',NS,1);

if strcmp(PutCall,'C')
    U = max(0, Si - Strike);
elseif strcmp(PutCall,'P')
    U = max(0, Strike- Si);
end

% Obtain the derivatives matrices
if strcmp(GridType,'Uniform')
    [derS derSS R] = BuildDerivativesUniformBS(S,T);
else
    [derS derSS R] = BuildDerivativesNonUniformBS(S,T);
end
I = eye(NS*NS);

% Build the matrices
L = (r-q).*derS + (0.5.*sigma.^2).*derSS - r.*R;
A = (I - thet.*dt.*L);
B = (I + (1-thet).*dt.*L);
if thet == 0;
    invA = I;
else
    if strcmp(method,'Inverse')
        invA = inv(A);
    end
end

% Loop through the time increments, updating U(t) to U(t+1) at each step
for t=2:NT
    u = U;
    
    % Explicit Method
    if thet==0
        U = B*u;
    end
    % Obtain U by calculating A inverse
    if strcmp(method,'Inverse')
        if thet==1
            U = invA*u;      % Implicit Method
        else
            U = (A\B)*u;       % Weighted Method
        end
    % Obtain U by Thomson method of solving Ax=b for A tridiagonal
    elseif strcmp(method,'Tridiagonal')
        U = MSolveTridiagonal(A,B*u);
    end

    % Early exercise for American options
    if strcmp(EuroAmer,'A')
        for s=1:NS*NS
            if strcmp(PutCall,'C')
                U(s,:) = max(U(s,:), Si(s) - Strike);
            elseif strcmp(PutCall,'P')
                U(s,:) = max(U(s,:), Strike - Si(s));
            end
        end
    end
end


% Retain only the first NS terms of the U vector
U1 = U(1:NS);

% Interpolate to get the final price
WPrice = interp1(S',U1,S0);
